﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;

namespace Models
{
    /// <summary>
    /// Classe qui gère la liaison entre Views et Models (View-Model-ViewModel).
    /// </summary>
    public class Manager : INotifyPropertyChanged
    {
        /// <summary>
        /// Liste des administrateurs de l'application.
        /// </summary>
        public List<Admin> Admins { get; private set; }

        /// <summary>
        /// Liste des utilisateurs de l'application.
        /// </summary>
        public List<Utilisateur> Utilisateurs { get; private set; }


        private ObservableCollection<Jeu> topRatedGames;
        /// <summary>
        /// Les jeux les mieux notés.
        /// </summary>
        public ObservableCollection<Jeu> TopRatedGames
        {
            get
            {
                return topRatedGames = new ObservableCollection<Jeu>(ListJeux.OrderByDescending(jeu => jeu.Note).Take(4));
            }
            set
            {
                topRatedGames = value;
                OnPropertyChanged();
            }
        }


        private ObservableCollection<Jeu> listJeuxAffiches;
        /// <summary>
        /// Les jeux affichés sur la page d'accueil.
        /// </summary>
        public ObservableCollection<Jeu> ListJeuxAffiches
        {
            get
            {
                var cinqPremiersJeux = ListJeux.Take(5);
                return listJeuxAffiches = new ObservableCollection<Jeu>(cinqPremiersJeux);
            }
            set
            {
                listJeuxAffiches = value;
                OnPropertyChanged();
            }
        }

        private ObservableCollection<Jeu> listJeux;
        /// <summary>
        /// Liste complète des jeux.
        /// </summary>
        public ObservableCollection<Jeu> ListJeux
        {
            get
            {
                return listJeux;
            }
            set
            {
                listJeux = value;
                OnPropertyChanged();
            }
        }

        /// <summary>
        /// Gestionnaire de persistance utilisé pour charger et sauvegarder les données.
        /// </summary>
        public IPersistanceManager Persistance { get; set; }

        private Utilisateur connectedUser;
        /// <summary>
        /// Utilisateur actuellement connecté.
        /// </summary>
        public Utilisateur? ConnectedUser
        {
            get => connectedUser;
            set
            {
                connectedUser = value;
                OnPropertyChanged();
            }
        }

        public event PropertyChangedEventHandler? PropertyChanged;

        /// <summary>
        /// Déclenche l'événement PropertyChanged pour une propriété spécifiée.
        /// </summary>
        /// <param name="propertyName">Le nom de la propriété modifiée.</param>
        public void OnPropertyChanged([CallerMemberName] string? propertyName = null)
            => PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));

        /// <summary>
        /// Constructeur de la classe Manager avec un gestionnaire de persistance spécifique.
        /// </summary>
        /// <param name="persistance">Le gestionnaire de persistance utilisé pour charger et sauvegarder les données.</param>
        public Manager(IPersistanceManager persistance)
        {
            Utilisateurs = new List<Utilisateur>();
            Admins = new List<Admin>();
            ListJeux = new ObservableCollection<Jeu>();
            Persistance = persistance;
            ConnectedUser = null;
        }

        /// <summary>
        /// Constructeur par défaut de la classe Manager.
        /// </summary>
        public Manager()
        {
            ListJeux = new ObservableCollection<Jeu>();
            Admins = new List<Admin>();
            Utilisateurs = new List<Utilisateur>();
            ConnectedUser = null;
        }

        /// <summary>
        /// Ajoute un jeu à la liste des jeux.
        /// </summary>
        /// <param name="jeux">Le jeu à ajouter.</param>
        public void AddJeux(Jeu jeux)
        {
            ListJeux.Add(jeux);
        }

        /// <summary>
        /// Ajoute un administrateur à la liste des administrateurs.
        /// </summary>
        /// <param name="admin">L'administrateur à ajouter.</param>
        public void AddAdmin(Admin admin)
        {
            Admins.Add(admin);
        }

        /// <summary>
        /// Ajoute un utilisateur à la liste des utilisateurs.
        /// </summary>
        /// <param name="utilisateur">L'utilisateur à ajouter.</param>
        public void AddUtilisateur(Utilisateur utilisateur)
        {
            Utilisateurs.Add(utilisateur);
        }

        /// <summary>
        /// Charge les données persistantes à partir du gestionnaire de persistance.
        /// </summary>
        public void ChargerDonnees()
        {
            var donnees = Persistance.ChargeDonnees();

            foreach (var user in donnees.Item3)
            {
                AddUtilisateur(user);
            }

            foreach (var admin in donnees.Item2)
            {
                AddAdmin(admin);
            }

            foreach (var jeu in donnees.Item1)
            {
                AddJeux(jeu);
            }
        }

        /// <summary>
        /// Sauvegarde les données persistantes en utilisant le gestionnaire de persistance.
        /// </summary>
        public void SauvegardeDonnees()
        {
            Persistance.SauvegardeDonnees(ListJeux, Admins, Utilisateurs);
        }
    }
}
